import scipy as SP
import h5py
import cPickle
import pdb

def load_arab107(phenotypes=['1_LD','2_LDV']):
    f = h5py.File('./../data/arab107/Nordborg_data.h5py','r')
    
    #1. genotypes
    X = f['geno/x'][:].T
    pos = f['geno/pos'][:]
    chrom = f['geno/chrom'][:]
    geno_ID    = SP.array(f['geno/meta'][1::,4],dtype='int')
    #2. phenotypes and matching
    phenotype_names = f['pheno/phenotype_names'][:]
    pheno_ID = f['pheno/ecotype_id'][:]
    Y  = f['pheno/Y'][:]

    #pdb.set_trace()
   
    #index of phenotypes
    Ip = SP.array([SP.nonzero(phenotype_names==phen)[0][0] for phen in phenotypes])
    #get mapping from all phenotypes to genotypes
    Igeno = SP.array([SP.nonzero(geno_ID==id)[0][0] for id in pheno_ID])
    
    #loop over selected phenotypes and create output dataset
    Ys = []
    Xs = []
    Traits = []
    Genotypes = [] 
    for ip in Ip:
        y = Y[:,ip]
        Iok = ~SP.isnan(y)
        y = y[Iok]
        igeno = Igeno[Iok]
        x = X[igeno,:]
        Ys.append(y[:,SP.newaxis])
        Xs.append(x)
        Traits.append(ip*SP.ones([y.shape[0],1]))
        Genotypes.append(igeno[:,SP.newaxis])
    pass
    RV = {'pos':pos,'chrom':chrom, 'TraitID': SP.concatenate(Traits,axis=0),'Geno': SP.concatenate(Xs,axis=0),'Pheno': SP.concatenate(Ys,axis=0),'GenoID':SP.concatenate(Genotypes,axis=0)}
    globals().update(RV)

    #add genic/ non-genic annotation
    annotation = cPickle.load(open('./../data/arab107/tair10cons_predicted_brief.pickle','rb'))   
    Igenic = SP.zeros([Geno.shape[1]],dtype='bool')
    #loop through annotatoin file
    for i in xrange(annotation.shape[0]):
        if (annotation[i,5]!='gene') | (annotation[i,6]!='mRNA'):
            continue 
        try: 
            _start = int(annotation[i,2])
            _stop  = int(annotation[i,3])
            _chrom = int(annotation[i,1])
        except Exception:
            continue
        ichrom = SP.nonzero(chrom==_chrom)[0]
        igene = (pos[ichrom]>=_start) & (pos[ichrom]<_stop)
        Igenic[ichrom[igene]] = True
        pass 

    RV['Igenic'] = Igenic
    #2. standardize phenotype
    Pheno = RV['Pheno'] - RV['Pheno'].mean(axis=0)
    Pheno /= Pheno.std(axis=0)
    RV['Pheno'] = Pheno
        
    #3. standardize Genotype
    X = SP.array(RV['Geno'],dtype='float')
    X -= X.mean(axis=0)
    X /= X.std(axis=0)

    Kgeno = 1.0*(RV['GenoID']==RV['GenoID'].T)

    RV['Kgeno'] = Kgeno
    RV['X'] = X
    RV['Kpop'] =  SP.dot(X,X.T)
    RV['Kpop_Igenic']  =  SP.dot(X[:,Igenic],X[:,Igenic].T)
    RV['Kpop_nIgenic'] = SP.dot(X[:,~Igenic],X[:,~Igenic].T)
    #4. convert trait Id to start from 0
    utrait = SP.unique(RV['TraitID'])
    TraitID = SP.zeros([RV['TraitID'].shape[0],1])
    for i in xrange(len(utrait)):
        II = (RV['TraitID']==utrait[i])
        TraitID[II] = i
    RV['TraitID'] = TraitID
    #5. calculate cumulative genomic position
    cum_pos = RV['pos'].copy()
    uchr = SP.unique(RV['chrom'])
    pmax_cumsum=0
    for i in xrange(uchr.shape[0]):
        i_chr = RV['chrom'] == uchr[i]
        pmax = RV['pos'][i_chr].max()
        cum_pos[i_chr] += pmax_cumsum
        pmax_cumsum+= pmax
    RV['cum_pos'] = cum_pos
    return RV


