import pylab as PL
import scipy as SP
import limix
import utils.preprocess as preprocess


def bar_plot_SW(R,field='LOD'):
    """bar plot of sliding window association results"""
    Y = R[field]
    X = R['pos']
    #loop
    for i in xrange(X.shape[0]):
        x0 = X[i,0]
        w  = X[i,1]-X[i,0]
        PL.bar(x0,Y[i],width=w,color='b')
    
def var_comp_SW(X,Pheno,pos,cum_pos,ws=1E6,sz=1E6,**kw_args):
    """calculate a sliding window based variance model
    ws: window size
    sz: step size
    """
    p  = 0
    hyperparams0 = None
    XX = SP.dot(X,X.T)
    #calculate on null model
    v0 = fit_variances(X,Pheno,SP.zeros([pos.shape[0]],dtype='bool'),XX,hyperparams0,**kw_args)
    LML0 = v0[1]
    #sliding window approach:
    LL = []
    while p<cum_pos.max():
        Iwindow = (cum_pos>(p-ws)) & (cum_pos< (p+ws))
        _rv = fit_variances(X,Pheno,Iwindow,XX,hyperparams0)
        _rv.insert(0,[cum_pos[Iwindow].min(),cum_pos[Iwindow].max()])
        LL.append(_rv)
        #move forward
        p += sz
        pass
    #collect
    RV = {}
    sx = SP.array([r[0] for r in LL])
    vs = SP.array([r[1] for r in LL])[:,:,0]
    LOD = SP.array([r[2] for r in LL]) -LML0
    RV['pos'] = sx
    RV['var'] = vs
    RV['var_block'] = vs[:,0]
    RV['LOD'] = LOD
    return RV

def fit_variances(X,y,Iwindow,XX=None,hyperparams0=None,background=True):
    """fit varaince components of an arbirary number of SNP subsets in snp_index and return
    X: genotyupes
    y: phenotype
    XX: SP.dot(X,X.T) precomputed (efficiency)
    hyperparams0: starting point for hyperparameter optimization
    background: condition on the genomic background (True)
    """
    covar = limix.CSumCF()
    #1. calculate covaraince of window
    XXwindow = SP.dot(X[:,Iwindow],X[:,Iwindow].T)
    XXgenome = XX-XXwindow
    if Iwindow.any():
        Kwindow = preprocess.scale_K(XXwindow)
    else:
        Kwindow = XXwindow
    Kgenome = preprocess.scale_K(XXgenome)
    covar_window = limix.CFixedCF(Kwindow)
    covar.addCovariance(covar_window)
    if background:
        covar_genome = limix.CFixedCF(Kgenome)
        covar.addCovariance(covar_genome)
    #construct Gaussian process model
    gp = limix.CGPbase(covar)
    gp.setY(y)
    #create parameters for optimization (start)
    if hyperparams0 is None:
        #create stupid starting points if not supplied:
        hyperparams0 = limix.CGPHyperParams()
        hyperparams0['covar'] = SP.zeros([covar.getNumberParams()])
        hyperparams0['lik']   = SP.log([0.01])
    gp.setParams(hyperparams0)
    gpopt = limix.CGPopt(gp)
    if 1:
        #add some constraints, not really needed but can be useful to avoid silly solutions
        constrainU = limix.CGPHyperParams()
        constrainL = limix.CGPHyperParams()
        constrainU['covar'] = +10*SP.ones_like(hyperparams0['covar'])
        constrainL['covar'] = -10*SP.ones_like(hyperparams0['covar'])
        constrainU['lik'] = +5*SP.ones_like(hyperparams0['lik'])
        constrainL['lik'] = -5*SP.ones_like(hyperparams0['lik'])
        gpopt.setOptBoundLower(constrainL);
        gpopt.setOptBoundUpper(constrainU);
    #optimize
    gpopt.opt()
    #get parameters and turn into varainces (they are on log scale)
    variances_covar =SP.exp(2*gp.getParams()['covar'])
    variances_lik   = SP.exp(2*gp.getParams()['lik'])
    
    total_variance = variances_covar.sum()+variances_lik.sum()
    #get margina likelihood
    LML = -1.0*gp.LML()
    return [variances_covar/total_variance,LML,gp.getParams()]
