
import sys
import cPickle
import os
#add limix path
#limix
import platform

#use is this as I am running code on linux and MAC.
#I have two compiled instances of limix around..
if len(platform.mac_ver()[0])>0:
    sys.path.append('./../lib/limix/build_mac/src/interfaces/python')
else:
    sys.path.append('./../lib/limix/build_linux/src/interfaces/python')
sys.path.append('./../lib/limix/python')
sys.path.append('./../lib')

#import main limix module
import limix
from utils.util import *
#import preprocessing utils
import utils.preprocess as preprocess

#pythong plotting and numerical tools
import scipy as SP
import pylab as PL
import h5py
import pdb
from plot import *
from qtl import *
import data


    
def getXfixed(TraitID):
    Ntrait = len(SP.unique(TraitID))
    Xfixed = SP.zeros([TraitID.shape[0],Ntrait])
    ii = 0
    for it in SP.unique(TraitID):
        Xfixed[:,ii] = 1.0*(TraitID==it)[:,0]
        ii+=1
    return Xfixed


if __name__ == '__main__':
 
    from pyplot import *
    PL.ion()
    
    pheno_names = ['5_FT10', '6_FT16']

    cache_file = './demo_cache.pickle'
    #1. load GWAS data
    if not os.path.exists(cache_file) or 'recalc' in sys.argv:
        #1. load data
        data = data.load_arab107(pheno_names)
        cPickle.dump(data,open(cache_file,'wb'),-1)
    else:
        data= cPickle.load(open(cache_file,'rb'))
    globals().update(data)

    #indicators for the two traits
    I0 = TraitID[:,0]==0
    I1 = TraitID[:,0]==1


    if 0:
        print "0. correlation analysis between traits"

        Y0 = Pheno[I0]
        Y1 = Pheno[I1]
        geno0 = GenoID[I0,0]
        geno1 = GenoID[I1,0]
        geno_map1 = SP.zeros([geno0.shape[0]],dtype='int')
        for i in xrange(geno0.shape[0]):
            g = geno0[i]
            if not g in geno1:
                geno_map1[i] = -1
            else:
                geno_map1[i] = SP.nonzero(geno1==g)[0][0]
        #filter
        Iok = (geno_map1>=0)
        geno_map1 = geno_map1[Iok]
        Y0 = Y0[Iok,0]
        Y1 = Y1[geno_map1,0]

        PL.figure()
        PL.plot(Y0,Y1,'k.')
        PL.xlabel('FT 10C')
        PL.ylabel('FT 16C')
        PL.savefig('./trait_correlation.pdf')
        pdb.set_trace()
    
    
    if 0:
        print "1. carrying out standard GWAs on both traits independently"
         
        #1. standard GWAs on each trait
        lm0 = simple_lmm(X[I0],Pheno[I0],Kpop[I0,:][:,I0])
        pv0 = lm0.getPv().ravel()
        
        lm1 = simple_lmm(X[I1],Pheno[I1],Kpop[I1,:][:,I1])
        pv1 = lm1.getPv().ravel()
        
        PL.figure(1,figsize=[12,6])
        PL.subplot(211)
        manhattanplot(pv0,chrom,pos)
        PL.title('10C')
        PL.xlabel('')
        PL.subplot(212)
        PL.title('16C')
        manhattanplot(pv1,chrom,pos)                

        PL.savefig('./gwa_10C_16C.pdf')
        PL.savefig('./gwa_10C_16C.png')
        
    if 1:            
        print "2. fit multi trait covariance model"
        pdb.set_trace()
        vt = limix.CMultiTraitVQTL()
        #trait index 
        vt.setTrait(TraitID)
        vt.setPheno(Pheno)
        #fixed effect:
        Xfixed = getXfixed(TraitID)
        vt.setFixed(Xfixed)
        #genotype identity
        vt.setKgeno(preprocess.scale_K(Kgeno))
        #add population structure
        vt.addK(preprocess.scale_K(Kpop))
        vt.train()
        opt = vt.getOpt()

        #plot covariance
        PL.figure()
        imagesc(vt.getCovar_term(0).K())
        PL.savefig('./mmt_covar.pdf')

        PL.figure()
        imagesc(vt.getCovar_noise().K())
        PL.savefig('./mmt_noise.pdf')

        
        #plot covariance matrices on consol
        print "Independent trait heritabilities"
        print "h0: %.2f" % (vt.estimateHeritability(Pheno[I0],Kpop[I0,:][:,I0]))
        print "h1: %.2f" % (vt.estimateHeritability(Pheno[I1],Kpop[I1,:][:,I1]))

        print "Full genotype covariance model"
        Gcovar =vt.getVarianceComponent_term(0)
        Gnoise =vt.getVarianceComponent_noise() 
        print "Genetic covariance matrix:"
        print Gcovar
        print "Noise covariance matrix:"
        print Gnoise
        print "Marginal heritabilities from joint analysis"
        print Gcovar.diagonal() / (Gcovar.diagonal() + Gnoise.diagonal())

        #get full covarinace matrix
        #rescale noise
        cn = vt.getCovar_noise()
        params=cn.getParams()
        Vp = SP.exp(2*params[[1,3]])
        Vp -= Vp.min()
        params[[1,3]] = 0.5*SP.log(Vp)
        Kmmt0 = vt.getCovar().K()
        cn.setParams(params)
        Kmmt = vt.getCovar().K()

        
    if 0:
        #varaince dissection in genic and non-genic SNPs
        print "2. fit multi trait covariance model splitting genic and non-genic components"    
        vt = limix.CMultiTraitVQTL()
        #trait index 
        vt.setTrait(TraitID)
        vt.setPheno(Pheno)
        #fixed effect:
        Xfixed = getXfixed(TraitID)
        vt.setFixed(Xfixed)
        #genotype identity
        vt.setKgeno(preprocess.scale_K(Kgeno))
        #add population structure
        vt.addK(preprocess.scale_K(Kpop_Igenic))
        vt.addK(preprocess.scale_K(Kpop_nIgenic))
        
        vt.train()
        opt = vt.getOpt()

        #plot covariance
        PL.figure()
        imagesc(vt.getCovar_term(0).K())
        Kfull = vt.getCovar().K()
        
        print "Full genotype covariance model"
        Gcovar0 =vt.getVarianceComponent_term(0)
        Gcovar1 =vt.getVarianceComponent_term(1)
        
        Gnoise =vt.getVarianceComponent_noise() 
        print "Genetic covariance matrix - genic"
        print Gcovar0
        
        print "Genetic covariance matrix - non-genic"
        print Gcovar1
        
        print "Noise covariance matrix:"
        print Gnoise
        print "Marginal heritabilities from joint analysis"
        print (Gcovar0+Gcovar1).diagonal() / ((Gcovar0 + Gcovar1).diagonal() + Gnoise.diagonal())
        pass
        
    if 0:
        #X = X[:,::10]
        print "3. use multit trait covariance for common effect analyses on both traits"
        lmcommon_simle  = simple_lmm(X,Pheno,Kpop,covs=Xfixed)
        pvcommon_simple = lmcommon_simle.getPv().ravel()
        
        lmcommon_mmt  = simple_lmm(X,Pheno,Kmmt,covs=Xfixed)
        pvcommon_mmt = lmcommon_mmt.getPv().ravel()
        
        #plot and compare
        PL.figure(figsize=[12,6])
        PL.subplot(211)
        manhattanplot(pvcommon_simple,chrom,pos)
        PL.xlabel('')
        PL.title('Mixed model')
        PL.subplot(212)
        PL.title('Multi-trait model')        
        manhattanplot(pvcommon_mmt,chrom,pos)
        
        PL.savefig('./lmm_multi_trait_simple.pdf')
        PL.savefig('./lmm_multi_trait_simple.png')
        
        
        
    
